/*--------------------------------------*/
/* 摜PSNRvZvO             */
/*                                      */
/*  Initial     2018/09/24   T.Kimura   */
/*                                      */
/*--------------------------------------*/
/* Copyright by T.Kimura 2018           */
/*--------------------------------------*/

/*----
  PSNR = 10*log(MAX^2)/MSE
       = 20*log(MAX/sqrt(MSE) )

  [mono]
  MSE = (1/(height*width))(I(x,y)-org(x,y))^2

  [color]
  r = Ir(x,y) - org_r(x,y)
  g = Ig(x,y) - org_g(x,y)
  b = Ib(x,y) - org_b(x,y)
  MSE = (1/{3*(height*width))(r^2+g^2+b^2)}
---*/
/*--------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "image.h"

#define		MONO_Depth		8		/* Mono color depth */
#define		COLOR_Depth		24		/* Full Color depth */
#define		MAX				255		/* 摜̉f蓾ől */

/*----------------------*/
/* Main Routine         */
/*----------------------*/
int main( int argv, char *argc[] )
{
    ImageData *img1/*Ǎ摜*/, *img2/*IWi摜*/;
    int 	x, y, res;
    Pixel 	pix1, pix2;
	double	d_r, d_g, d_b, d_mono;
	double  mse;
	double  max;
	double	psnr;

    /* ꍇ */
    if( argv < 3 ) {
        printf("p[^܂\n");
        printf("psnr <read image file name> <Original image file name>\n");
        return 1;
    }

    // t@CǍ摜f[^̓ǂݍ
    res=readBMPfile( argc[1]/*read image*/, &img1);
    if( res<0 ) {
        printf("]Ώۂ̉摜f[^ǂ߂܂");
        return 1;
    }

    // t@CIWi摜f[^̓ǂݍ
    res=readBMPfile( argc[2]/*read image*/, &img2);
    if( res<0 ) {
        printf("IWi摜f[^ǂ߂܂");
        return 1;
    }

	// 摜TCYE摜ނ̊mF
	if( (img1->height) != (img2->height)
	 || (img1->width)  != (img2->width)
	 || (img1->depth)  != (img2->depth)  ) {
		printf("Q̉摜̃TCY܂͎ނقȂ܂\n");
		return 1;
	}
	else {
		if( img1->depth == MONO_Depth ) {
			printf("r摜̓mNł\n");
		}
		else {
			printf("r摜̓J[ł\n");
		}
	}

    /*------------------------------------------------------------------*/
    /* Ǎ摜f[^ img1 ɓCIWi摜f[^ img2 ɓ */
    /*------------------------------------------------------------------*/
	mse = 0.0;
    for( y=0 ; y<img1->height ; y++ ){
        for( x=0 ; x<img1->width ; x++ ) {
            getPixel(img1, x, y, &pix1);
            getPixel(img2, x, y, &pix2);

			if( (img1->depth)==COLOR_Depth ) {
				d_r = pix1.r - pix2.r;
				d_g = pix1.g - pix2.g;
				d_b = pix1.b - pix2.b;
				mse += (d_r*d_r + d_g*d_g + d_b*d_b);
			}
			else {
				d_r = pix1.r - pix2.r;
				mse += (d_r * d_r);
			}
        }
    }

	if( (img1->depth)==COLOR_Depth ) {
		mse = mse / ( 3 * img1->height * img1->width );
	}
	else {
		mse = mse / (img1->height * img1->width);
	}
	psnr = 20 * log10( MAX / sqrt(mse) );

    // t@C։摜f[^o
    disposeImage(img1);
    disposeImage(img2);

	printf("PSNR -> %lf dB, MSE -> %lf\n", psnr, mse );

    return 0;
}

